/* Copyright (C) 2018 Thomas Balzer */

/* This file is part of tomd. */

/* tomd is free software: you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or */
/* (at your option) any later version. */

/* tomd is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with tomd.  If not, see <http://www.gnu.org/licenses/>. */

#include <stdlib.h>
#include <stdio.h>
#include <sys/un.h>
#include <sys/socket.h>
#include <unistd.h>

#include "../../include/macros.h"

static int sfd;

static void header(void)
{
  tomc_p("Tom's Client, Copyright (C) 2018 Thomas Balzer");
  tomc_p("GPL v3 or later license.");
}

static void init(void)
{
  /* init socket connection */
  sfd =
    socket(PF_LOCAL,
           SOCK_STREAM,
           0);
  if(sfd < 0){
    perror("socket");
    exit(EXIT_FAILURE);
  }

  struct sockaddr_un addr;
  addr.sun_family = AF_LOCAL;
  sprintf(addr.sun_path, "/run/user/1000/tomd/socket");
  
  if(connect(sfd, (struct sockaddr *) &addr, SUN_LEN(&addr)) != 0){
    perror("connect");
    exit(EXIT_FAILURE);
  }
}

#define HALLO "hallo there from dumb-client"

void write_hallo(void)
{
  printf("writing hallo\n");
  ssize_t wrote =
    write(sfd, HALLO, sizeof HALLO);
  printf("wrote %d bytes.\n", wrote);
}

static char *default_name = "no_name";
static struct{
  char status, stop, kill, start;
  char  *name;
} options;

static void extract_options(int argc, char **argv)
{
  /* -status <name>  Get status - if we thought we ran it, current real status */
  /* -stop <name>    Run provided command to gently stop */
  /* -kill <name>    Aggressive killing of process (signal 15) */
  /* -start <name>   Run name */
  if(argc == 1){
    /* only tomc given */
    tomc_p("no args given.");
    exit(EXIT_SUCCESS);
  }

  options.name = NULL;
  
  for(int i = 1;
      i < argc;
      i++){
    char *arg = argv[i];
#define X(op) {if(strcmp("-" #op, arg) == 0){options.op = 1; continue;}}
    X(status);
    X(stop);
    X(kill);
    X(start);
#undef X
    /* assume last non option was the name */
    options.name = arg;
  }

  if(options.name == NULL){
    options.name = default_name;
  }
}

static void print_options()
{
  tomc_p("---------");
  tomc_p("  name: %s", options.name);
  tomc_p("  kill: %d", options.kill);
  tomc_p("status: %d", options.status);
  tomc_p("  stop: %d", options.stop);
  tomc_p(" start: %d", options.start);
  tomc_p("---------");
}

int main(int argc, char **argv)
{
  header();
  extract_options(argc, argv);
  print_options();
  init();
  
  
  return EXIT_SUCCESS;
}
