/* Copyright (C) 2018 Thomas Balzer */

/* This file is part of tomd. */

/* tomd is free software: you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or */
/* (at your option) any later version. */

/* tomd is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with tomd.  If not, see <http://www.gnu.org/licenses/>. */

#include <stddef.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/un.h>
#include <sys/stat.h>
#include <sys/socket.h>

#include "../../include/macros.h"
#include "../../include/manifest.h"

static void header(void)
{
  tomd_p("Tom's Daemon, Copyright (C) 2018 Thomas Balzer");
  tomd_p("GPL v3 or later license.");
}

static int sfd;
static char socket_dirname[100];
static char socket_filename[100];

static void gen_socket_filename(void)
{
  uid_t this_user_id =
    getuid();
  sprintf(socket_dirname,
          "/run/user/%d/tomd",
          this_user_id);
  sprintf(socket_filename,
          "/run/user/%d/tomd/socket",
          this_user_id);

  tomd_p("making dir '%s'", socket_dirname);
  /* lazy way */
  char buf[100];
  sprintf(buf, "mkdir -p %s", socket_dirname);
  system(buf);
  /* if(mkdir(socket_dirname, S_IRWXU) != 0){ */
  /*   perror("socket mkdir"); */
  /*   exit(EXIT_FAILURE); */
  /* } */

  tomd_p("removing file '%s'", socket_filename);
  if(unlink(socket_filename) != 0){
    if(errno == ENOENT){
      tomd_p("socket file doesn't exist");
    }else{
      perror("[tomd] socket unlink");
      exit(EXIT_FAILURE);
    }
  }
}

static void socket_init(void)
{
  gen_socket_filename();
  
  /* listen on socket, respond to requests. */
  /* perform all registered 'tick' operations */
  sfd =
    socket(PF_LOCAL,         /* namespace - local unix socket */
           SOCK_STREAM,      /* style */
           0);               /* protocol */

  if(sfd < 0){
    perror("[tomd] socket");
    exit(EXIT_FAILURE);
  }

  struct sockaddr_un name;
  size_t size;

  name.sun_family = AF_LOCAL;
  strncpy(name.sun_path,
          socket_filename,
          sizeof(name.sun_path));

  size = SUN_LEN(&name);
  tomd_p("attempting to bind to '%s'",
         socket_filename);
  if(bind(sfd,
          (struct sockaddr *) &name,
          size) < 0){
    perror("[tomd] bind");
    exit(EXIT_FAILURE);
  }

  tomd_p("initialized tomd socket connections");
}

static void init(void)
{
  socket_init();
  load_jobs();
}

/* These defines are here to improve the local readability */
#define CHDIR            0
#define DONT_CHDIR       1
#define CLOSE_PIPES      0
#define DONT_CLOSE_PIPES 1

static void daemonize(void)
{
  /* daemon is in unistd. */
  /* arg1 - 0 changes dir to / */
  /* arg2 - 0 closes all pipes (/dev/null) */
  /* WARNING > THIS BEHAVIOR IS SILENT AND EASY TO MISPLACE THE
     PROCESS  */
  daemon(DONT_CHDIR, DONT_CLOSE_PIPES);
}

static void run(void)
{
  /* loop: */
  /*  1 listen */
  /*  2 accept/select */
  /*  3 handle dgram */
  int listen_bool =
    listen(sfd,
           10); /* max connection limit is 10 for arbitrary reasons */
  if(listen_bool == -1){
    perror("[tomd] listen");
    exit(EXIT_FAILURE);
  }

  for(;;){
    struct sockaddr addr;
    socklen_t size;
    int accept_sfd =
      accept(sfd,
             &addr,             /* requester info */
             &size);            /* len of requester info */
    if(accept_sfd < 0){
      perror("[tomd] accept socket");
      exit(EXIT_FAILURE);
    }

    /* int getsockname(accept_sfd, */
    /*                 addr, */
    /*                 size); */
    
    tomd_p("accepted socket connection from '%s'",
           ((struct sockaddr_un *) &addr)->sun_path);

    char accept_buf[100] = {};
    
    read(accept_sfd, accept_buf, 100);
    tomd_p("got message '%s', accept_buf");
    close(accept_sfd);
  }
}

static void cleanup(void)
{
  if(sfd >= 0){
    close(sfd);
  }
}

int main(int argc, char **argv)
{
  header();
  init();
  /* daemonize(); */
  /* run(); */
  run_jobs();
  cleanup();
  
  return EXIT_SUCCESS;
}
